local vanillaBuildings = require("prototypes/buildings/UpdatesVanilla")

local biterFunctions = require("prototypes/utils/BiterUtils")

local swarmUtils = require("prototypes/SwarmUtils")

if settings.startup["rampant-newEnemies"].value then
    swarmUtils.processFactions()
end

if settings.startup["rampant-removeBloodParticles"].value then
    local explosions = data.raw["explosion"]

    for k,v in pairs(explosions) do
        if string.find(k, "blood") then
            v["created_effect"] = nil
        end
    end
end

if settings.startup["rampant-unitSpawnerBreath"].value then
    for _, unitSpawner in pairs(data.raw["unit-spawner"]) do
        if (string.find(unitSpawner.name, "hive") or string.find(unitSpawner.name, "biter") or
            string.find(unitSpawner.name, "spitter")) then
            if not unitSpawner.flags then
                unitSpawner.flags = {}
            end
            unitSpawner.flags[#unitSpawner.flags+1] = "breaths-air"
        end
    end
end


for k, unit in pairs(data.raw["unit"]) do
    if (string.find(k, "biter") or string.find(k, "spitter")) and unit.collision_box then
        if settings.startup["rampant-enableSwarm"].value then
            unit.collision_box = {
                {unit.collision_box[1][1] * 0.20, unit.collision_box[1][2] * 0.20},
                {unit.collision_box[2][1] * 0.20, unit.collision_box[2][2] * 0.20}
            }
        end

        unit.affected_by_tiles = settings.startup["rampant-unitsAffectedByTiles"].value

        unit.ai_settings = {
            destroy_when_commands_fail = false,
            allow_try_return_to_spawner = true,
            path_resolution_modifier = -5,
            do_seperation = true
        }
    end
end

if settings.startup["rampant-enableShrinkNestsAndWorms"].value then
    for k, unit in pairs(data.raw["unit-spawner"]) do
        if (string.find(k, "biter") or string.find(k, "spitter") or string.find(k, "hive")) and unit.collision_box then
            unit.collision_box = {
                {unit.collision_box[1][1] * 0.50, unit.collision_box[1][2] * 0.50},
                {unit.collision_box[2][1] * 0.50, unit.collision_box[2][2] * 0.50}
            }
        end
    end

    for k, unit in pairs(data.raw["turret"]) do
        if string.find(k, "worm") and unit.collision_box then
            unit.collision_box = {
                {unit.collision_box[1][1] * 0.50, unit.collision_box[1][2] * 0.50},
                {unit.collision_box[2][1] * 0.50, unit.collision_box[2][2] * 0.50}
            }
        end
    end
end


if settings.startup["rampant-addWallResistanceAcid"].value then
    vanillaBuildings.addWallAcidResistance()
end
-- dampant-alienmodule-compat
local loot_to_add = {"artifact-ore"}

function DampantAddLootToCategory(category, probability_per_tier, c_min, c_max)
  for name, table_entry in pairs(data.raw[category]) do
    v, t = string.match(name, "%-v(%d+)%-t(%d+)%-rampant")
    if v ~= nil and t ~= nil then
      if table_entry.loot == nil then
        table_entry.loot = {}
      end
      for _, loot in pairs(loot_to_add) do
        local already_has_loot = false
        for _, loot_entry in pairs(table_entry.loot) do
          if loot_entry.item == loot then
            already_has_loot = true
          end
        end
        if not already_has_loot then
          p = math.min(probability_per_tier * tonumber(t), 1)
          table.insert(table_entry.loot, {
            item = loot,
            probability = p,
            count_min = c_min,
            count_max = c_max,
          })
          -- log("added loot " .. loot .. " (" .. p .. "," .. c_min .. "," .. c_max .. ") to " .. name)
        else
          -- log("skipping loot " .. loot .. " for " .. name)
        end
      end
    end
  end
end


if settings.startup["dampant-alienmodule-compat-enabled"].value then
    DampantAddLootToCategory("unit",
        settings.startup["dampant-alienmodule-compat-probability-per-tier-unit"].value,
        settings.startup["dampant-alienmodule-compat-min-count-unit"].value,
        settings.startup["dampant-alienmodule-compat-max-count-unit"].value)
    DampantAddLootToCategory("turret",
        settings.startup["dampant-alienmodule-compat-probability-per-tier-turret"].value,
        settings.startup["dampant-alienmodule-compat-min-count-turret"].value,
        settings.startup["dampant-alienmodule-compat-max-count-turret"].value)
    DampantAddLootToCategory("unit-spawner",
        settings.startup["dampant-alienmodule-compat-probability-per-tier-spawner"].value,
        settings.startup["dampant-alienmodule-compat-min-count-spawner"].value,
        settings.startup["dampant-alienmodule-compat-max-count-spawner"].value)
end
